@@ -2,12 +2,11 @@ package CGI::Header;
 use 5.008_009;
 use strict;
 use warnings;
-use overload q{""} => 'as_string', bool => 'SCALAR', fallback => 1;
 use Carp qw/carp croak/;
 use List::Util qw/first/;
 use Scalar::Util qw/blessed/;
 
-our $VERSION = '0.35';
+our $VERSION = '0.40';
 
 our $MODIFY = 'Modification of a read-only value attempted';
 
@@ -61,15 +60,6 @@ sub normalize_field_name {
     croak "'-$field' can't be used as a field name";
 }
 
-sub lc {
-    carp "The lc() method is obsolete and will be removed in 0.36";
-    my $class = shift;
-    my $str = CORE::lc shift;
-    $str =~ s/^-//;
-    $str =~ tr/-/_/;
-    $str;
-}
-
 sub time2str {
     require CGI::Util;
     CGI::Util::expires( $_[1], 'http' );
@@ -160,7 +150,7 @@ my %GET = (
     },
     p3p => sub {
         my $self = shift;
-        my $tags = join ' ', $self->p3p_tags;
+        my $tags = join ' ', $self->p3p;
         $tags ? qq{policyref="/w3c/p3p.xml", CP="$tags"} : undef;
     },
     pragma => sub {
@@ -173,11 +163,15 @@ my %GET = (
     },
     set_cookie => sub {
         my $self = shift;
-        $self->{header}->{-cookie};
+        $self->{header}->{-cookie} || undef;
+    },
+    status => sub {
+        my ( $self, $prop ) = @_;
+        $self->{header}->{$prop} || undef;
     },
     window_target => sub {
         my $self = shift;
-        $self->{header}->{-target};
+        $self->{header}->{-target} || undef;
     },
 );
 
@@ -217,7 +211,7 @@ my %SET = (
         carp "Can't assign to '-expires' directly, use expires() instead";
     },
     p3p => sub {
-        carp "Can't assign to '-p3p' directly, use p3p_tags() instead";
+        carp "Can't assign to '-p3p' directly, use p3p() instead";
     },
     pragma => sub {
         my ( $self, $prop, $value ) = @_;
@@ -265,6 +259,14 @@ my %EXISTS = (
         my ( $self, $prop ) = @_;
         $self->_has_date or exists $self->{header}->{$prop};
     },
+    expires => sub {
+        my ( $self, $prop ) = @_;
+        $self->{header}->{$prop};
+    },
+    p3p => sub {
+        my ( $self, $prop ) = @_;
+        $self->{header}->{$prop};
+    },
     pragma => sub {
         my ( $self, $prop ) = @_;
         $self->query->cache or exists $self->{header}->{$prop};
@@ -275,11 +277,15 @@ my %EXISTS = (
     },
     set_cookie => sub {
         my $self = shift;
-        exists $self->{header}->{-cookie};
+        $self->{header}->{-cookie};
+    },
+    status => sub {
+        my ( $self, $prop ) = @_;
+        $self->{header}->{$prop};
     },
     window_target => sub {
         my $self = shift;
-        exists $self->{header}->{-target};
+        $self->{header}->{-target};
     },
 );
 
@@ -321,6 +327,7 @@ my %DELETE = (
         my ( $self, $prop ) = @_;
         delete $self->{header}->{-cookie};
     },
+    status => '_delete',
     window_target => sub {
         my ( $self, $prop ) = @_;
         delete $self->{header}->{-target};
@@ -346,6 +353,7 @@ sub _delete {
 }
 
 sub is_empty {
+    carp "'is_empty' is obsolete and will be removed in 0.41";
     !$_[0]->SCALAR;
 }
 
@@ -362,6 +370,29 @@ sub clone {
     ref( $self )->new( \%copy, $self->{query} );
 }
 
+sub _push {
+    my ( $self, $prop, @values ) = @_;
+
+    if ( my $value = $self->{header}->{$prop} ) {
+        return push @{$value}, @values if ref $value eq 'ARRAY';
+        unshift @values, $value;
+    }
+
+    $self->{header}->{$prop} = @values > 1 ? \@values : $values[0];
+
+    scalar @values;
+}
+
+sub push_p3p {
+    my $self = shift;
+    $self->_push( '-p3p', @_ );
+}
+
+sub push_cookie {
+    my $self = shift;
+    $self->_push( '-cookie', @_ );
+}
+
 BEGIN {
     my @conflicts = (
         attachment => [ '-content_disposition' ],
@@ -388,6 +419,28 @@ BEGIN {
     }
 }
 
+BEGIN {
+    my @props = qw(
+        charset
+        location
+        status
+        target
+        type
+    );
+
+    for my $prop ( @props ) {
+        my $code = sub {
+            my $self = shift;
+            return $self->{header}->{$prop} unless @_;
+            $self->{header}->{$prop} = shift;
+            $self;
+        };
+
+        no strict 'refs';
+        *{$prop} = $code;
+    }
+}
+
 sub nph {
     my $self   = shift;
     my $header = $self->{header};
@@ -403,23 +456,52 @@ sub nph {
     $NPH or $header->{-nph};
 }
 
-sub p3p_tags {
+sub cookie {
+    my $self = shift;
+
+    if ( @_ ) {
+        $self->{header}->{-cookie} = @_ > 1 ? [ @_ ] : shift;
+    }
+    elsif ( my $cookie = $self->{header}->{cookie} ) {
+        return ref $cookie eq 'ARRAY' ? @{$cookie} : $cookie;
+    }
+    else {
+        return;
+    }
+
+    $self;
+}
+
+sub p3p {
     my $self   = shift;
     my $header = $self->{header};
 
     if ( @_ ) {
         $header->{-p3p} = @_ > 1 ? [ @_ ] : shift;
     }
-    elsif ( my $tags = $header->{-p3p} ) {
-        return ref $tags eq 'ARRAY' ? @{$tags} : split ' ', $tags;
+    elsif ( my $p3p = $header->{-p3p} ) {
+        my @tags = ref $p3p eq 'ARRAY' ? @{$p3p} : $p3p;
+        return map { split ' ', $_ } @tags;
+    }
+    else {
+        return;
     }
 
-    return;
+    $self;
+}
+
+sub p3p_tags {
+    carp "p3p_tags() is obsolete and will be removed in 0.41";
+    my $self = shift;
+    $self->p3p( @_ );
+}
+
+sub as_hashref {
+    +{ $_[0]->flatten };
 }
 
 sub flatten {
     my $self  = shift;
-    my $level = defined $_[0] ? int shift : 2;
     my $query = $self->query;
     my %copy  = %{ $self->{header} };
     my $nph   = delete $copy{-nph} || $query->nph;
@@ -439,9 +521,8 @@ sub flatten {
     }
 
     if ( $cookie ) {
-        my @cookies = $level && ref $cookie eq 'ARRAY' ? @{$cookie} : $cookie;
-           @cookies = map { "$_" } @cookies if $level > 1;
-        push @headers, map { ('Set-Cookie', $_) } @cookies;
+        my @cookies = ref $cookie eq 'ARRAY' ? @{$cookie} : $cookie;
+        push @headers, map { ('Set-Cookie', "$_") } @cookies;
     }
 
     push @headers, 'Expires', $self->time2str($expires) if $expires;
@@ -467,6 +548,8 @@ sub flatten {
 sub each {
     my ( $self, $callback ) = @_;
 
+    carp "'each' is obsolete and will be removed in 0.41";
+
     if ( ref $callback eq 'CODE' ) {
         my @headers = $self->flatten;
         while ( my ($field, $value) = splice @headers, 0, 2 ) {
@@ -480,7 +563,10 @@ sub each {
     $self;
 }
 
-sub field_names { keys %{{ $_[0]->flatten(0) }} }
+sub field_names {
+    carp "'field_names' is obsolete and will be removed in 0.41";
+    keys %{{ $_[0]->flatten }};
+}
 
 sub as_string {
     my $self = shift;
@@ -505,7 +591,7 @@ sub SCALAR {
 
 sub FIRSTKEY {
     my $self = shift;
-    my @fields = $self->field_names;
+    my @fields = keys %{ $self->as_hashref };
     ( $self->{iterator} = sub { shift @fields } )->();
 }
 
@@ -562,7 +648,7 @@ CGI::Header - Adapter for CGI::header() function
 
 =head1 VERSION
 
-This document refers to CGI::Header version 0.35.
+This document refers to CGI::Header version 0.40.
 
 =head1 DEPENDENCIES
 
@@ -825,6 +911,8 @@ This will remove all header fields.
 
 =item $bool = $header->is_empty
 
+This method is obsolete and will be removed in 0.41.
+
 Returns true if the header contains no key-value pairs.
 
   $header->clear;
@@ -859,6 +947,8 @@ In this case, the outgoing header will be formatted as:
 
 =item $header->p3p_tags( @tags )
 
+This method will be renamed to C<p3p> in 0.41.
+
 Represents P3P tags. The parameter can be an array or a space-delimited
 string. Returns a list of P3P tags. (In scalar context,
 returns the number of P3P tags.)
@@ -901,6 +991,8 @@ with a NPH (no-parse-header) script.
 
 =item @fields = $header->field_names
 
+This method is obsolete and will be removed in 0.41.
+
 Returns the list of distinct field names present in the header
 in a random order.
 The field names have case as returned by C<CGI::header()>.
@@ -910,6 +1002,8 @@ The field names have case as returned by C<CGI::header()>.
 
 =item $self = $header->each( \&callback )
 
+This method is obsolete and will be removed in 0.41.
+
 Apply a subroutine to each header field in turn.
 The callback routine is called with two parameters;
 the name of the field and a value.
@@ -941,18 +1035,7 @@ Returns pairs of fields and values.
   #     ...
   # )
 
-  $header->flatten(1);
-  # => (
-  #     "Set-Cookie" => $cookie1,
-  #     "Set-Cookie" => $cookie2,
-  #     ...
-  # )
-
-  $header->flatten(0);
-  # => (
-  #     "Set-Cookie" => [$cookie1, $cookie2],
-  #     ...
-  # )
+=item $header->as_hashref
 
 =item $header->as_string
 
@@ -994,13 +1077,6 @@ You can also iterate through the tied hash:
 
 See also L<perltie>.
 
-=head2 OVERLOADED OPERATORS
-
-The following operators are L<overload>ed:
-
- ""   -> as_string
- bool -> SCALAR
-
 =head1 EXAMPLES
 
 =head2 WRITING Blosxom PLUGINS
@@ -1097,7 +1173,7 @@ You can't assign to the P3P header directly:
 
 C<CGI::header()> restricts where the policy-reference file is located,
 and so you can't modify the location (C</w3c/p3p.xml>).
-You're allowed to set P3P tags using C<p3p_tags()>.
+You're allowed to set P3P tags using C<p3p()>.
 
 =item Pragma
 
